//
//
//

#include "cordnumfunc.h"

#include <lpmd/matrix.h>
#include <lpmd/util.h>
#include <lpmd/simulation.h>

#include <sstream>

using namespace lpmd;

CordNumFunc::CordNumFunc(std::string args): Plugin("cordnumfunc", "2.0")
{
 ParamList & params = (*this);
 //
 DefineKeyword("start", "0");
 DefineKeyword("end", "-1");
 DefineKeyword("each", "1");
 DefineKeyword("rcut","10");
 DefineKeyword("output","cordnumfunc-output.dat");
 DefineKeyword("bins", "200");
 ProcessArguments(args);
 cut = double(params["rcut"]);
 nb = int(params["bins"]);
 start = int(params["start"]);
 end = int(params["end"]);
 each = int(params["each"]);
 OutputFile() = params["output"];
}

CordNumFunc::~CordNumFunc() { }

void CordNumFunc::ShowHelp() const
{
 std::cout << "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n";
 std::cout << " Module Name        = cordnumfunc                                              \n";
 std::cout << " Problems Report to = admin@lpmd.cl                                            \n";
 std::cout << "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n";
 std::cout << " General Info      >>                                                          \n";
 std::cout << "      This plugin is used to calculate the coordination number (CN) of an      \n";
 std::cout << "      atomic configuration. The CN is calculated between the neighbors of the  \n";
 std::cout << "      subcells generated by the linked cell method.                            \n";
 std::cout << "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n";
 std::cout << " General Options   >>                                                          \n";
 std::cout << "      bins          : Sets the number of subdivisions of the range [0,rcut].   \n";
 std::cout << "      rcut          : Set the cutoff radius for the function.                  \n";
 std::cout << "      output        : Output file.                                             \n";
 std::cout << "      average       : Sets if the the property must be averaged over all       \n";
 std::cout << "                      configurations (true / false)                            \n";
 std::cout << "      start         : Determines in which step the plugin begins to be applied.\n";
 std::cout << "      end           : Determines in which step the plugin ceases to be applied.\n";
 std::cout << "      each          : Determines how often (each how many time-steps) the      \n";
 std::cout << "                      plugin must be applied.                                  \n";
 std::cout << "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n";
 std::cout << " Example           >>                                                          \n";
 std::cout << " #Loading the plugin :                                                         \n";
 std::cout << " use cordnumfunc                                                               \n";
 std::cout << "     bins 200                                                                  \n";
 std::cout << "     rcut 10.0                                                                 \n";
 std::cout << "     output cordnumf.dat                                                       \n";
 std::cout << "     average false                                                             \n";
 std::cout << " enduse                                                                        \n";
 std::cout << " #Applying the plugin :                                                        \n";
 std::cout << " property cordnumfunc start=0 each=1 end=100                                 \n\n";
 std::cout << "      The plugin is used to calculate the CN of the atomic configuration every \n";
 std::cout << "      step of the first 100 steps. The data is written in the file cordnumf.dat\n";
 std::cout << "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n";
}

void CordNumFunc::Evaluate(lpmd::Configuration & con, Potential & pot)
{
 assert(&pot != 0);//icc 869
 lpmd::BasicParticleSet & atoms = con.Atoms();
 lpmd::Array <int> esp = atoms.Elements();
 if (nb <= 0 || atoms.Size() <=0) throw PluginError("cordnumfunc", "Error in calculation.");
 int nsp = esp.Size();
 unsigned long int N = atoms.Size();
 double **histo;
 double **cnfun;
 histo = new double*[nsp*nsp];
 cnfun = new double*[nsp*nsp];
 for(int i=0;i<nsp*nsp;i++) { histo[i]=new double[nb]; }
 for(int i=0;i<nsp*nsp;i++) { cnfun[i]=new double[nb];}
 for (int i=0;i<nsp*nsp;i++)
 { 
  for (int j=0;j<nb;j++) histo[i][j]=0.0e0;
  for (int j=0;j<nb;j++) cnfun[i][j]=0.0e0;
 }
 double rcut=cut;
 double dr=rcut/nb;

 
 lpmd::Array<std::string> pairs;
 for(int i=0; i<esp.Size() ; ++i)
 {
  for(int j=0; j<esp.Size() ; ++j)
  {
   std::ostringstream ostr;
   ostr << ElemSym[esp[i]]<< "-" << ElemSym[esp[j]];
   pairs.Append(ostr.str());
  }
 }
 int s=0;
 for(int i=0;i<pairs.Size();++i)	   
 {
  //Hace funcional cada una de las especies de los pares.
  lpmd::Array<std::string> loa = StringSplit(pairs[i],'-'); // lista de atomos
  int e1 = ElemNum(loa[0]);
  int e2 = ElemNum(loa[1]); 
  //Cuenta los atomos de la especie 1.
  int ne1=0;
  for (unsigned long int j=0;j<N;j++) {if (atoms[j].Z()==e1) ne1++;}
  //Comienzan las iteraciones.
  for (unsigned long int j=0;j<N;j++)
  {
   if(atoms[j].Z()==e1)
   {
    lpmd::NeighborList & nlist = con.Neighbors(j,true,rcut);
    for(long int k=0;k<nlist.Size();++k)
    {
     const lpmd::AtomPair & nn = nlist[k];
     if(nn.j->Z()==e2)
     {
      if(nn.r2<rcut*rcut)
      {
       int in=(long)floor(sqrt(nn.r2)/dr);
       histo[s][in]++;
      }
     }
    }
   }
  }
  for(int k=0;k<nb;++k)
  {
   for(int j=0;j<k;j++)
   {
    cnfun[s][k]+=(histo[s][j]/ne1);
   }
  }
  s++;
 }

 //
 // Output of cordnum - histogram format
 //
 Matrix & m = CurrentValue(); 
 m = lpmd::Matrix(1 + nsp*nsp, nb);
 // Asigna los labels al objeto Matrix para cada columna
 m.SetLabel(0, "r");
 int j=1;
 for (int i=0;i<pairs.Size();++i)
 {
  m.SetLabel(j, pairs[i]+" cn");
  j++;
 }
 //
 double k=0.0e0;
 for(int i=0;i<nb;i++)
 {
  m.Set(0, i, k);
  for(j=0;j<(int)(nsp*nsp);j++)
  {
   m.Set(j+1, i, cnfun[j][i]);
  }
  k+=dr;
 }
 //Borra arreglos dinamicos.
 for(int i=0;i<nsp*nsp;i++)
 {
  delete[] cnfun[i];
  delete[] histo[i];
 }
 delete [] cnfun;
 delete [] histo;
}

// Esto se incluye para que el modulo pueda ser cargado dinamicamente
Plugin * create(std::string args) { return new CordNumFunc(args); }
void destroy(Plugin * m) { delete m; }

